package com.hzya.frame.nifiapi.service;

import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.hzya.frame.nifiapi.client.NifiClient;
import com.hzya.frame.nifiapi.model.joinbindparametercontexts.BindParameterContextsJoin11;
import com.hzya.frame.nifiapi.model.joincontrollerenabled.EnOrDiControllerServices12;
import com.hzya.frame.nifiapi.model.joincreateconnection.CreateConnection18;
import com.hzya.frame.nifiapi.model.joincreateconnections.CreateConnections18;
import com.hzya.frame.nifiapi.model.joincreatetemp.CreateTemplateJoin;
import com.hzya.frame.nifiapi.model.joingetcontroller.ControllerService12;
import com.hzya.frame.nifiapi.model.joinparametercontexts.ParameterContextsJoin;
import com.hzya.frame.nifiapi.model.joinprocessgroups.Component7;
import com.hzya.frame.nifiapi.model.joinprocessgroups.Position7;
import com.hzya.frame.nifiapi.model.joinprocessgroups.ProcessGroupsJoin;
import com.hzya.frame.nifiapi.model.joinsnippetinstance.SnippetInstanceJoin;
import com.hzya.frame.nifiapi.model.joinsnippets.SnippetsJoin;
import com.hzya.frame.nifiapi.model.joinstartorstopprocessgroup.StartOrStopProcessGroupsInfoJoin10;
import com.hzya.frame.nifiapi.model.joinupdateprocessor.RunStatusOrStop17;
import com.hzya.frame.nifiapi.model.nifitemplates.NifiTemplates;
import com.hzya.frame.nifiapi.model.processgroupid.ProcessGroupsId;
import com.hzya.frame.nifiapi.model.processgrouproot.ProcessGroupsRoot;
import com.hzya.frame.nifiapi.model.resultcreateprocessor.Component15;
import com.hzya.frame.nifiapi.model.resultcreateprocessor.CreateProcessorJoin15;
import com.hzya.frame.nifiapi.model.resultcreateprocessor.Position15;
import com.hzya.frame.nifiapi.model.resultcreateprocessors.CreateProcess16;
import com.hzya.frame.nifiapi.model.resultcreatesnippet.SnippetInstance15;
import com.hzya.frame.nifiapi.model.resultcreatetemplate.NewNifiTemplatete14;
import com.hzya.frame.nifiapi.model.resultparametercontexts.ParameterContextsResult;
import com.hzya.frame.nifiapi.model.resultprocessgroups.ProcessgroupsResult;
import com.hzya.frame.nifiapi.model.resultprocessgroupsinfo.ProcessGroupsInfoResult9;
import com.hzya.frame.nifiapi.model.resultprocessorsinfo.ProcessorsInfo16;
import com.hzya.frame.nifiapi.model.resultsnippets.SnippetResult13;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * nifi api 服务类
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.nifi.service
 * @Project：fw-nifi
 * @name：NifiService
 * @Date：2025/5/14 10:02
 * @Filename：NifiService
 */
@Service
public class NifiApiService {

    private final NifiClient client;

    private final ObjectMapper objectMapper;

    @Autowired
    public NifiApiService(NifiClient client, ObjectMapper objectMapper) {
        this.client = client;
        this.objectMapper = objectMapper;
    }

    /**
     * 获取token
     */
    public String getAccessToken() throws Exception {
        return client.getAccessToken();
    }

    /**
     * 查询全部流程
     */
    public ProcessGroupsRoot queryFlowProcessGroupsRoot() throws Exception {
        return client.get("/flow/process-groups/root", ProcessGroupsRoot.class);
    }

    /**
     * 查询指定流程
     */
    public ProcessGroupsId queryFlowProcessGroupsRoot(String flowGroupId) throws Exception {
        return client.get(StrUtil.format("/flow/process-groups/{}", flowGroupId), ProcessGroupsId.class);
    }

    /**
     * 创建参数上下文
     */
    public ParameterContextsResult createParameterContexts(ParameterContextsJoin parameterContextsJoin) throws Exception {
        return client.post("/parameter-contexts", parameterContextsJoin, ParameterContextsResult.class);
    }

    /**
     * 更新参数上下文
     */
    public ParameterContextsResult updateParameterContexts(ParameterContextsResult parameterContextsJoin) throws Exception {
        return client.post(StrUtil.format("/parameter-contexts/{}/update-requests", parameterContextsJoin.getId()), parameterContextsJoin, ParameterContextsResult.class);
    }

    /**
     * 查询参数上下文
     */
    public ParameterContextsResult getParameterContexts(String paramContextsId) throws Exception {
        return client.get(StrUtil.format("/parameter-contexts/{}", paramContextsId), ParameterContextsResult.class);
    }

    /**
     * 删除参数上下文
     */
    public void deleteParameterContexts(String paramContextsId) throws Exception {
        client.delete(StrUtil.format("/parameter-contexts/{}", paramContextsId), ParameterContextsResult.class);
    }

    /**
     * 删除参数上下文
     */
    public ParameterContextsResult deleteParameterContexts(String id, Map<String, String> queryParams) throws Exception {
        return client.delete(StrUtil.format("/parameter-contexts/{}", id), queryParams, ParameterContextsResult.class);
    }

    /**
     * 查询流程组详情
     */
    public ProcessGroupsInfoResult9 getProcessGroups(String processGroups) throws Exception {
        return client.get(StrUtil.format("/process-groups/{}", processGroups), ProcessGroupsInfoResult9.class);
    }

    /**
     * 创建流程组
     */
    public ProcessgroupsResult createProcessGroups(String parentGroupId, ProcessGroupsJoin processGroupsJoin) throws Exception {
        Component7 component = processGroupsJoin.getComponent();
        if (component != null && component.getPosition() == null) {
            double[] doubles = calculatePosition(parentGroupId);
            component.setPosition(new Position7(String.valueOf(doubles[0]), String.valueOf(doubles[1])));
        }
        return client.post(StrUtil.format("/process-groups/{}/process-groups", parentGroupId), processGroupsJoin, ProcessgroupsResult.class);
    }

    /**
     * 删除流程组
     */
    public ProcessGroupsInfoResult9 deleteProcessGroups(String parentGroupId, Map<String, String> queryParams) throws Exception {
        return client.delete(StrUtil.format("/process-groups/{}", parentGroupId), queryParams, ProcessGroupsInfoResult9.class);
    }

    /**
     * 启动或停止流程组
     */
    public ProcessGroupsInfoResult9 startOrStopProcessGroups(StartOrStopProcessGroupsInfoJoin10 startOrStopProcessGroupsInfoJoin10) throws Exception {
        return client.put(StrUtil.format("/flow/process-groups/{}", startOrStopProcessGroupsInfoJoin10.getId()), startOrStopProcessGroupsInfoJoin10, ProcessGroupsInfoResult9.class);
    }

    /**
     * 给流程组绑定参数上下文环境、或者给流程组更新上下文环境
     */
    public ProcessGroupsInfoResult9 bindParameterContexts(BindParameterContextsJoin11 bindParameterContextsJoin) throws Exception {
        String id = bindParameterContextsJoin.getComponent().getId();
        return client.put(StrUtil.format("/process-groups/{}", id), bindParameterContextsJoin, ProcessGroupsInfoResult9.class);
    }

    /**
     * 查询控制器服务详情
     */
    public ControllerService12 getControllerServices(String controllerServicesId) throws Exception {
        return client.get(StrUtil.format("/controller-services/{}", controllerServicesId), ControllerService12.class);
    }

    /**
     * 创建流程组组级别的控制器服务,控制器服务分为流程组级别和全局级别，其中流程组级别只能在流程组内的处理器共享
     *
     * @param id                  对应的流程组id
     * @param controllerService11 提交参数
     */
    public ControllerService12 createControllerServices(String id, ControllerService12 controllerService11) throws Exception {
        return client.post(StrUtil.format("/process-groups/{}/controller-services", id), controllerService11, ControllerService12.class);
    }

    /**
     * 启用或禁用控制器服务
     */
    public ControllerService12 enabledOrDisabledControllerServices(String id, EnOrDiControllerServices12 enOrDiControllerServices) throws Exception {
        return client.put(StrUtil.format("/controller-services/{}/run-status", id), enOrDiControllerServices, ControllerService12.class);
    }

    /**
     * 删除控制器服务
     */
    public ControllerService12 deleteControllerServices(String id, Map<String, String> queryParam) throws Exception {
        return client.delete(StrUtil.format("/controller-services/{}", id), queryParam, ControllerService12.class);
    }

    /**
     * 获取所有xml模板
     */
    public NifiTemplates getAllXmlTemplates() throws Exception {
        return client.get("/flow/templates", NifiTemplates.class);
    }

    /**
     * 根据模板id，获取xml模板内容
     */
    public InputStream getXmlTemplatesContent(String templatesId) throws Exception {
        return client.getAsStream(StrUtil.format("/templates/{}/download", templatesId));
    }

    /**
     * 把指定的流程组生成模板
     */
    public NewNifiTemplatete14 createNifiTemplate(String id, CreateTemplateJoin createTemplateJoin) throws Exception {
        return client.post(StrUtil.format("/process-groups/{}/templates", id), createTemplateJoin, NewNifiTemplatete14.class);
    }

    /**
     * 将某个流程组复制为代码片段
     */
    public SnippetResult13 createSnippets(SnippetsJoin snippetsJoin) throws Exception {
        return client.post("/snippets", snippetsJoin, SnippetResult13.class);
    }

    /**
     * 将代码片段实例化到指定位置
     */
    public SnippetInstance15 createSnippetInstance(String groupId, SnippetInstanceJoin snippetInstanceJoin) throws Exception {
        if (snippetInstanceJoin.getOriginX() == null || snippetInstanceJoin.getOriginY() == null) {
            double[] doubles = calculatePosition(groupId);
            snippetInstanceJoin.setOriginX(String.valueOf(doubles[0]));
            snippetInstanceJoin.setOriginY(String.valueOf(doubles[1]));
        }
        return client.post(StrUtil.format("/process-groups/{}/snippet-instance", groupId), snippetInstanceJoin, SnippetInstance15.class);
    }

    /**
     * 创建处理器，nifi1.28.1版本有364个处理器，并且每个入参都有差异，所以创建之前，可以先查询一遍处理器得到对应的参数，进行修改调整
     */
    public CreateProcess16 createProcessor(String groupId, CreateProcessorJoin15 createProcessorJoin15) throws Exception {
        Component15 component = createProcessorJoin15.getComponent();
        if (component != null && component.getPosition() == null) {
            double[] doubles = calculatePosition(groupId);
            component.setPosition(new Position15(String.valueOf(doubles[0]), String.valueOf(doubles[1])));
        }
        return client.post(StrUtil.format("/process-groups/{}/processors", groupId), createProcessorJoin15, CreateProcess16.class);
    }

    /**
     * 查询处理器详情
     */
    public ProcessorsInfo16 getProcessor(String processId) throws Exception {
        return client.get(StrUtil.format("/processors/{}", processId), ProcessorsInfo16.class);
    }

    /**
     * 更新处理器参数，如果处理器正在运行则无法更新
     */
    public ProcessorsInfo16 updateProcessor(String processId, ProcessorsInfo16 processorsInfo16) throws Exception {
        return client.put(StrUtil.format("/processors/{}", processId), processorsInfo16, ProcessorsInfo16.class);
    }

    /**
     * 启用/停止处理器
     */
    public ProcessorsInfo16 runStatusOrStop(String processId, RunStatusOrStop17 runStatusOrStop17) throws Exception {
        return client.put(StrUtil.format("/processors/{}/run-status", processId), runStatusOrStop17, ProcessorsInfo16.class);
    }

    /**
     * 删除处理器详情，如果处理器正在运行则无法删除
     */
    public ProcessorsInfo16 deleteProcessor(String processId, Map<String, String> queryParams) throws Exception {
        return client.delete(StrUtil.format("/processors/{}", processId), queryParams, ProcessorsInfo16.class);
    }

    /**
     * 建立连接关系
     */
    public CreateConnection18 createProcessorConnections(String processGroupsId, CreateConnections18 createConnections17) throws Exception {
        return client.post(StrUtil.format("/process-groups/{}/connections", processGroupsId), createConnections17, CreateConnection18.class);
    }

    /**
     * 计算新处理器或流程组的x、y坐标
     *
     * @param parentGroupId 父流程组ID
     * @return 包含X和Y坐标的数组
     * @throws Exception 如果获取流程组信息失败
     */
    public double[] calculatePosition(String parentGroupId) throws Exception {
        String jsonString = client.getRaw(StrUtil.format("/flow/process-groups/{}", parentGroupId));
        JsonNode json = objectMapper.readTree(jsonString);

        //查找processGroupFlow.flow
        JsonNode flow = json.path("processGroupFlow").path("flow");

        List<double[]> positions = new ArrayList<>();
        String[] componentTypes = {"processors", "processGroups", "funnels", "remoteProcessGroups", "inputPorts", "outputPorts", "labels"};

        // 遍历所有组件类型,收集坐标
        for (String type : componentTypes) {
            JsonNode array = flow.path(type);
            if (array.isArray()) {
                for (JsonNode entity : array) {
                    JsonNode position = entity.path("component").path("position");
                    if (position.isMissingNode()) {
                        position = entity.path("position"); // 某些组件如端口可能直接包含 position
                    }
                    if (position.has("x") && position.has("y")) {
                        double x = position.get("x").asDouble();
                        double y = position.get("y").asDouble();
                        positions.add(new double[]{x, y});
                    }
                }
            }
        }

        // 如果没有子组件，代表流程组为null，返回默认坐标
        if (positions.isEmpty()) {
            return new double[]{100.0, 100.0};
        }

        // 计算新坐标：基于最下方的组件向下偏移
        double maxY = Double.MIN_VALUE;
        double xAtMaxY = 0;
        for (double[] pos : positions) {
            if (pos[1] > maxY) {
                maxY = pos[1];
                xAtMaxY = pos[0];
            }
        }
        double newX = xAtMaxY;      // 与最下方组件保持相同X坐标
        double newY = maxY + 260; // 向下偏移260像素
        return new double[]{newX, newY};
    }
}